// Copyright 2011 Joe Ferner <joe.ferner@nearinfinity.com>
#include "./connection.h"

node_db_sqlite::Connection::Connection()
    : connection(NULL) {
}

node_db_sqlite::Connection::~Connection() {
    this->close();
}

void node_db_sqlite::Connection::setFilename(const std::string& filename) throw() {
    this->filename = filename;
}

void node_db_sqlite::Connection::open() throw(node_db::Exception&) {
    this->close();

    int rc = sqlite3_open(this->filename.c_str(), &this->connection);
    if(rc != SQLITE_OK) {
        std::stringstream err;
        err << "Cannot open SQLite database: " << rc;
        this->alive = false;
        throw node_db::Exception(err.str());
    }
    this->alive = true;
}

void node_db_sqlite::Connection::close() {
    if (this->alive) {
        sqlite3_close(this->connection);
    }
    this->alive = false;
}

node_db::Result* node_db_sqlite::Connection::query(const std::string& query) const throw(node_db::Exception&) {
    sqlite3_stmt *stmt;
    int rc = sqlite3_prepare_v2(this->connection, query.c_str(), query.length(), &stmt, NULL);
    if (rc != SQLITE_OK) {
        std::stringstream err;
        err << "Cannot execute statement: " << rc;
        throw node_db::Exception(err.str());
    }

    return new node_db_sqlite::Result(this->connection, stmt);
}

void replace(std::string& target, const std::string oldstr, const std::string newstr) {
    unsigned int x = 0;
    while(x = target.find(oldstr, x), x != std::string::npos) {
        target.erase(x, oldstr.length());
        target.insert(x, newstr);
        x += newstr.length();
    }
}

std::string node_db_sqlite::Connection::escape(const std::string& string) const throw(node_db::Exception&) {
    std::string result = string;
    replace(result, "\"", "\\\"");
    replace(result, "'", "\\'");
    return result; // TODO: what should we escape here?
}

std::string node_db_sqlite::Connection::version() const {
    std::string version = std::string(SQLITE_VERSION);
    return version;
}
