// Copyright 2011 Joe Ferner <joe.ferner@nearinfinity.com>
#include "./sqlite.h"

v8::Persistent<v8::FunctionTemplate> node_db_sqlite::SQLite::constructorTemplate;

node_db_sqlite::SQLite::SQLite(): node_db::Binding() {
    this->connection = new node_db_sqlite::Connection();
    assert(this->connection);
}

node_db_sqlite::SQLite::~SQLite() {
    if (this->connection != NULL) {
        delete this->connection;
    }
}

void node_db_sqlite::SQLite::Init(v8::Handle<v8::Object> target) {
    v8::HandleScope scope;

    v8::Local<v8::FunctionTemplate> t = v8::FunctionTemplate::New(New);

    constructorTemplate = v8::Persistent<v8::FunctionTemplate>::New(t);
    constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);

    node_db::Binding::Init(target, constructorTemplate);

    target->Set(v8::String::NewSymbol("SQLite"), constructorTemplate->GetFunction());
}

v8::Handle<v8::Value> node_db_sqlite::SQLite::New(const v8::Arguments& args) {
    v8::HandleScope scope;

    node_db_sqlite::SQLite* binding = new node_db_sqlite::SQLite();
    if (binding == NULL) {
        THROW_EXCEPTION("Can't create client object")
    }

    if (args.Length() > 0) {
        ARG_CHECK_OBJECT(0, options);

        v8::Handle<v8::Value> set = binding->set(args[0]->ToObject());
        if (!set.IsEmpty()) {
            return scope.Close(set);
        }
    }

    binding->Wrap(args.This());

    return scope.Close(args.This());
}

v8::Handle<v8::Value> node_db_sqlite::SQLite::set(const v8::Local<v8::Object> options) {
    ARG_CHECK_OBJECT_ATTR_OPTIONAL_STRING(options, filename);

    node_db_sqlite::Connection* connection = static_cast<node_db_sqlite::Connection*>(this->connection);

    v8::String::Utf8Value filename(options->Get(filename_key)->ToString());

    if (options->Has(filename_key)) {
        connection->setFilename(*filename);
    }

    return v8::Handle<v8::Value>();
}

v8::Persistent<v8::Object> node_db_sqlite::SQLite::createQuery() const {
    v8::Persistent<v8::Object> query(
        node_db_sqlite::Query::constructorTemplate->GetFunction()->NewInstance());
    return query;
}
