/* Escape & Query building tests */
try {
    nodeunit = require("nodeunit");
} catch(err) {
    require.paths.unshift("/usr/lib/node_modules");
    nodeunit = require("nodeunit/lib/nodeunit");
}
var testCase = nodeunit.testCase;
var fs = require("fs");
var async = require("async");

var settings = JSON.parse(require('fs').readFileSync('./tests-settings.json','utf8'));

function createDbClient(callback) {
    new sqlite.Database(settings).connect(function(err) {
        if (err) {
            throw new Error('Could not connect to test DB');
        }
        callback(this);
    });
}

var sqlite = require("./db-sqlite");
var tests = require("./lib/node-db/tests.js").get(createDbClient);

tests["sqlite"] = testCase({
    "setUp": function(callback) {
        var self = this;
        fs.unlink(settings.filename, function(err) {
            if(err) { console.trace(err); throw err; }
            createDbClient(function(client) {
                self.client = client;
                client.query("CREATE TABLE users (id INTEGER PRIMARY KEY ASC, name TEXT)").execute(function(err, rows) {
                    if(err) { console.trace(err); throw err; }
                    callback();
                });
            });
        });
    },
    "insert, retreive, delete": function(test) {
        var client = this.client;

        async.series([
            function(done) {
                client.query("INSERT INTO users (id, name) VALUES (NULL, 'bob')").execute(function(err, rows) {
                    test.ifError(err);
                    test.equals(rows.affected, 1);
                    done();
                });
            },
            function(done) {
                client.query("INSERT INTO users (id, name) VALUES (NULL, 'joe')").execute(function(err, rows) {
                    test.ifError(err);
                    test.equals(rows.affected, 1);
                    done();
                });
            },
            function(done) {
                client.query().select('*').from('users').order({'name': { order: true }}).execute(function(err, rows) {
                    test.ifError(err);
                    test.equals(rows.length, 2);
                    test.equals(rows[0].name, 'bob');
                    test.equals(rows[1].name, 'joe');
                    done();
                });
            },
            function(done) {
                client.query().delete().from("users").execute(function(err, rows) {
                    test.ifError(err);
                    test.equals(rows.affected, 2);
                    done();
                });
            },
            function(done) {
                client.query().select('*').from('users').order({'name': { order: true }}).execute(function(err, rows) {
                    test.ifError(err);
                    test.equals(rows.length, 0);
                    done();
                });
            }
        ], function() { test.done(); });
    }
});

for(var test in tests) {
    exports[test] = tests[test];
}
